#ifndef CONTIG_H
#define CONTIG_H

#include "header.h"
#include "shorty-string.h"

class ReadProcessor;
//class ShortyString;

// Defines a data structure that helps to print a contig info in .contig format.
class Contig {
 public:
  //  Contig(ReadProcessor* rp);
  Contig(bool right, int seed_id, int seed_len, const ShortyString& str,
	 const ReadProcessor* rp);
  Contig(bool right, int seed_id, int seed_len, const ShortyString& str,
	 int id, const ReadProcessor* rp);
  Contig(const ShortyString& str);

  Contig SubContig(int start, int len) const;
  void PrintContigFormat(fstream* out) const;
  void PrintFastaFormat(fstream* out) const;
  void PrintGeoFormat(fstream* out) const;
  bool IsGoodSeed(fstream* out, const Contig& c) const;
  void AddRead(int id, int offset, int gen, mar_types mt);

  inline void SetStr(const ShortyString& str) { str_ = str; }
  inline void SetId(int id) { id_ = id; }
  inline int GetId() { return id_; }
  inline const ShortyString& Str() const { return str_; }
  inline int Size() const { return reads_.size(); }
  inline int GetRead(int i) const { return reads_[i]; }
  inline int GetOffset(int i) const { return offsets_[i]; }
  //  inline int GetSeedOffset(int i) const  { return seed_offsets_[i]; }
  inline mar_types GetMt(int i) const { return mar_types_[i]; }
  inline int GetMinGen() const { return min_gen_; }
  inline int GetMaxGen() const { return max_gen_; }
  inline int GetMinOffset() const { return min_offset_; }
  inline int GetMaxOffset() const { return max_offset_; }
  inline int GetMedOffset() const { return med_offset_; }

  // Add a value to all the offsets.
  void ShiftOffsets(int x);

  // Add another contig with this contig. Note that we have to set str_
  // seperately.
  void Add(const Contig& c);

  IntPair GapWindow() const;

 private:

  // 'true' if it is right to its seed, 'false' otherwise.
  bool right_;

  // Id of the contig. Integer for the time being.
  int id_;
  int seed_id_, seed_len_;

  // Min and max offsets of the reads. 'med_offset_' is a placeholder.
  int min_offset_, max_offset_, med_offset_;

  // Min and max gen of the reads.
  int min_gen_, max_gen_;

  // This is the actual contig.
  ShortyString str_;

  // List of identifiers and offsets of the reads that formed this contig.
  // 'seed_offsets_' is the offset of corresponding mar in the seed.
  IntVec reads_, offsets_;//, seed_offsets_;

  // Mar types of the reads.
  MtVec mar_types_;

  // This will help us to fetch the reads when we need them.
  const ReadProcessor* rp_;
};

#endif  //  CONTIG_H
