#include "header.h"

// Defines the hash table structure.
class Hash {
 public:
  Hash(int len);
  
  ~Hash();

  // Getting hash value for 'hash_len_' characters starting from 'start'.
  // The selected substring is converted to a 4-base number. It returns
  // -1 if there is an error (e.g. invalid 'start', or invalid character
  // in 'str'.
  int HashVal(const string& str, int start) const;

  // CAUTION: due to many changes, this method might be wrong. VERIFY.
  // This is same as 'HashVal' in terms of functionality. But if we want 
  // to calculate the hash value of a left shifted, previously hashed string,
  // we can calculate the new hash value easily with less computation. Its
  // return value will be similar as 'HashVal'. 'gone' is the first character in
  // the previous string, 'come' is the new chanracter at the end, 'prev' is the
  // old hash value.
  int LeftShiftHashVal(char gone, char come, int prev) const;
  
  // Used to insert 'str' into this hash table. We insert two keys,
  // usually they are (id, offset). Hashing is performed from the 
  // position 'start' for 'hash_len_' characters.
  // It returns -2 if the entry is already there, hash value if inserted, 
  // -1 if there is an error.
  int Insert(const string& str, int keya, int keyb, int start);

  // Same as before except we do incremental hashing using 'LeftShiftHashVal'.
  int Insert(const string& str, int keya, int keyb, int start, int prev);
  // Get the id'th bucket from 'table_'.
  const list<IntPair>& GetBucket(int id) const;
 private:
  // This is the hash table. We store a pair of values (as described before)
  // for each candidate.
  vector<list<IntPair> > table_;
  
  // **
  //  vector<vector<int> > key_;

  // Number of characters from the beginning used to hash a string.
  int hash_len_;

  // Assigns an int value to a char. Currently we are only interested
  // about A, T, G, C, a, t, g, c.
  int char_val_[256];
};
