#ifndef TRIE_H
#define TRIE_H

#include "header.h"

class Trie {
 public:
  Trie() {}
  void Insert(const string& str, int id, mar_types mt);
  IntMtPair Search(const string& str) const;
  IntMtPairVec Search(const string& str, int offset, int error_allowed) const;
  static int node_count_;

 private:
  Trie(const Trie&);
  class ListNode{
  public:
    ListNode() { next_ = NULL; }
    ListNode(int id, mar_types mt)
      : id_(id), mt_(mt) { next_ = NULL;}
    void Insert(int id, mar_types mt) {
      ListNode* t = new ListNode(id, mt);
      if (next_) {
	t->next_ = next_->next_;
	next_->next_ = t;
      } else {
	next_ = t;
      }
    }
    int id_;
    mar_types mt_;    
    ListNode* next_;
    
  private:
  };
  class TrieNode {
  public:
    TrieNode() {
      for (int i = 0; i < BASE_TYPES; ++i) {
	children_[i] = 0;
      }
      leaf_info_ = 0;
    }
    ~TrieNode() {
      for (int i = 0; i < BASE_TYPES; ++i) {
	if (children_[i]) delete children_[i];
      }
      if (leaf_info_) delete leaf_info_;
    }
    //int mer_id_;
    TrieNode* children_[BASE_TYPES];
    //    IntMtPairVec* leaf_info_;
    ListNode* leaf_info_;

  private:
    TrieNode(const TrieNode&);
  };
  IntMtPair VisitNode(const string& str, int pos, const TrieNode* node) const;
  void VisitNode(const string& str, int pos, int rem_error,
		 const TrieNode* node, IntMtPairVec* ret_info) const;
  TrieNode root_;
};

#endif  // TRIE_H
