#include <algorithm>
#include <cmath>
#include <cstdlib>
#include <fstream>

#include "visualize.h"
#include "viz-contig.h"

Visualize::Visualize(const string& seq, fstream* fout)
  :original_seq(seq) {
  coverage = (int *)malloc(sizeof(int) * seq.length());
  maxrow = (int *)malloc(sizeof(int) * seq.length());

  // Initialize all coverage to zero.
  for (int i = 0; i < seq.length(); ++i) {
    coverage[i] = maxrow[i] = 0;
  }
  fout_ = fout;
}

Visualize::~Visualize() {
  free(coverage);
  free(maxrow);
}

void Visualize::AddContig(const string& seq, int offset) {
  VizContig ctg(seq, 0, offset);  // We will set the row later on.
  vector<int> ms;
  for (int i = 0; i < contigs_.size(); ++i) {
    int s1 = offset;
    int e1 = s1 + seq.length() - 1;
    int s2 = contigs_[i].col();
    int e2 = s2 + contigs_[i].contig().length() - 1;
    if ((s2 >= s1 && s2 <= e1) ||
	(e2 >= s1 && s2 <= e1) ||
	(s1 >= s2 && s1 <= e2) ||
	(e1 >= s2 && e1 <= e2)) {
      ms.push_back(contigs_[i].row());
    }
  }
  sort(ms.begin(), ms.end());
  int m = 0;
  if (ms.size() == 0 || ms[0] != 1) m = 1;
  else {
    m = ms[ms.size() - 1] + 1;
    for (int i = 0; i < ms.size() - 1; ++i) {
      if (ms[i + 1] - ms[i] > 1) {
	m = ms[i] + 1;
	break;
      }
    }
  }
  for (int i = offset; i < offset + seq.length(); ++i) {
    ++coverage[i];
  }
  ctg.setRow(m);
  for (int i = offset; i < offset + seq.length(); ++i) {
    maxrow[i] = m;
  }
  contigs_.push_back(ctg);
  //printf("%d-%d-%s\n", ctg.row(), ctg.col(), ctg.contig().c_str());
}

void Visualize::Print() {
  // Sort the contigs according to row, then column.
  sort(contigs_.begin(), contigs_.end(), CompareContig());
  //  for (int i = 0; i < contigs_.size(); ++i) {
  //  printf("%d-%d-%s\n", contigs_[i].row(), contigs_[i].col(), contigs_[i].contig().c_str());
  //}
  int prev_col = 0;
  for (int i = 0; i < contigs_.size(); ++i) {
    if (i && contigs_[i].row() != contigs_[i - 1].row()) {
      //printf("\n");
      *fout_ << endl;
      prev_col = 0;
    }
    for (int j = 0; j < contigs_[i].col() - prev_col; ++j) {
      //printf(" ");
      *fout_ << " ";
    }
    //printf("%s", contigs_[i].contig().c_str());
    for (int j = 0; j < contigs_[i].contig().length(); ++j) {
      char ch = contigs_[i].contig()[j];
      if (ch != original_seq[contigs_[i].col() + j]) {
	*fout_ << (char)((ch != 'N' && ch != '?' && ch != '*' && ch != '+'
			  && (ch < 'a' || ch > 'z')) ? ch + ' ' : ch);
	--coverage[contigs_[i].col() + j];
      } else {
	//printf("%c", ch);
	*fout_ << ch;
      }
      //printf("%c", ch != original_seq[contigs_[i].col() + j] ? ch + ' ' : ch);
    }
    prev_col = contigs_[i].col() + contigs_[i].contig().length();
  }

  // Fix this...
  // printf("\n");
  *fout_ << endl;
  for (int i = 0; i < original_seq.length(); ++i) {
    //printf("-");
    *fout_ << "-";
  }
  //printf("\n%s\n", original_seq.c_str());
  *fout_ << endl << original_seq << endl;

  int max_c = 0; // Maximum coverage.
  //int min_c = contigs_.size(); // Minimum coverage.
  for (int i = 0; i < original_seq.length(); ++i) {
    max_c = max(max_c, coverage[i]);
    //min_c <?= coverage[i];
  }
  //printf("max_c = %d\n", max_c);
  for (int i = 0; i < original_seq.length(); ++i) {
    if (max_c) {
      *fout_ << (int)(floor((9.0 / max_c) * coverage[i]));  
    } else {
      //      printf("0");
      *fout_ << "0";
    }
  }
  //  printf("\n");
  *fout_ << endl;

  // Print position info along the x-axis.
  for (int i = 0; i < original_seq.length(); ++i) {
    
    // In future, we can parameterize this. 
    *fout_ << (i % 10 ? " " : ".");
  }
  //printf("\n");
  *fout_ << endl;
  for (int i = 0; i < original_seq.length(); ++i) {
    if (i != original_seq.length() - 1) {
      if (i % 10) {
	//printf(" ");
	*fout_ << " ";
      } else {
	//printf("%d", i);
	*fout_ << i;
	int temp = i / 10;
	while (temp) {
	  ++i;
	  temp /= 10;
	}
      }
    }
  }
  //printf("\n");
  *fout_ << endl;
  *fout_ << "maximum coverage: " <<  max_c << endl;
}

// void Visualize::PrintHtml() {
//   printf("<html><head><title></title></head><body>\n");
//   // Sort the contigs according to row, then column.
//   sort(contigs_.begin(), contigs_.end(), CompareContig());
//   //  for (int i = 0; i < contigs_.size(); ++i) {
//   //  printf("%d-%d-%s\n", contigs_[i].row(), contigs_[i].col(), contigs_[i].contig().c_str());
//   //}
//   int prev_col = 0;
//   for (int i = 0; i < contigs_.size(); ++i) {
//     if (i && contigs_[i].row() != contigs_[i - 1].row()) {
//       printf("\n");
//       prev_col = 0;
//     }
//     for (int j = 0; j < contigs_[i].col() - prev_col; ++j) {
//       printf("<font face=courier>&nbsp;</font>");
//     }
//     //printf("%s", contigs_[i].contig().c_str());
//     for (int j = 0; j < contigs_[i].contig().length(); ++j) {
//       char ch = contigs_[i].contig()[j];
//       if (ch != original_seq[contigs_[i].col() + j]) {
// 	printf("<font face=courier color=red>%c</font>", (ch != 'N' && ch != '?') ? ch + ' ' : ch);
// 	--coverage[contigs_[i].col() + j];
//       } else {
// 	printf("<font face=courier color=green>%c</font>", ch);
//       }
//       //printf("%c", ch != original_seq[contigs_[i].col() + j] ? ch + ' ' : ch);
//     }
//     prev_col = contigs_[i].col() + contigs_[i].contig().length();
//   }

//   // Fix this...
//   printf("\n");
//   for (int i = 0; i < original_seq.length(); ++i) {
//     printf("<font face=courier>-</font>");
//   }
//   //  printf("\n<font face=courier>%s</font>\n", original_seq.c_str());
  
//   int max_c = 0; // Maximum coverage.
//   //int min_c = contigs_.size(); // Minimum coverage.
//   for (int i = 0; i < original_seq.length(); ++i) {
//     max_c >?= coverage[i];
//     //min_c <?= coverage[i];
//   }
//   //printf("max_c = %d\n", max_c);
//   printf("\n");
//   for (int i = 0; i < original_seq.length(); ++i) {
//     printf("<font face=courier>%c</font>", original_seq[i]);
//   }
//   printf("\n");
//   for (int i = 0; i < original_seq.length(); ++i) {
//     if (max_c) printf("<font face=courier>%d</font>", (int)(floor((9.0 / max_c) * coverage[i]))); 
//     else printf("<font face=courier>0</font>");
//   }
//   printf("\n");

//   // Print position info along the x-axis.
//   for (int i = 0; i < original_seq.length(); ++i) {
    
//     // In future, we can parameterize this. 
//     printf(i % 10 ? "<font face=courier>&nbsp;</font>" : "<font face=courier>.</font>");
//   }
//   printf("\n");
//   for (int i = 0; i < original_seq.length(); ++i) {
//     if (i != original_seq.length() - 1) {
//       if (i % 10) {
// 	printf("<font face=courier>&nbsp;</font>");
//       } else {
// 	printf("<font face=courier>%d</font>", i);
// 	int temp = i / 10;
// 	while (temp) {
// 	  ++i;
// 	  temp /= 10;
// 	}
//       }
//     }
//   }
//   printf("\n");
//   printf("<font face=courier>maximum coverage: %d</font>\n", max_c);
//   printf("</body></html>");
// }

int Visualize::similarity(const string&a, const string& b) {
  
}
