#include <string>
#include <vector>

using namespace std;

class VizContig;

class Visualize {
 public:
  // Constructor. 'seq' is set to 'sequence'.
  Visualize(const string& seq, fstream* fout);
  ~Visualize();

  // Adds a contig for processing, it will find the appropriate row, column
  // for 'contig'. 'offset' denotes the possible position of 'contig'
  // in 'original_seq'.
  void AddContig(const string& contig, int offset);

  // Print the contigs in aligned fashion. This should be the last method to
  // invoke as it modifies some information, e.g. 'coverage'.
  void Print();

  // Same as 'print', but prints in html format -- Not functional at this point
  void PrintHtml();
 private:
  // Calculate the similarity between 'a' and 'b'. At this point, it is
  // extremely simple.
  int similarity(const string& a, const string& b);

  // Coverage of a point in 'original_seq'.
  int *coverage;

  // Maximum row values given at this point
  int *maxrow;

  // This is the actual sequence
  string original_seq;

  // These are the contigs.
  vector<VizContig> contigs_;

  // Output stream.
  fstream* fout_;
};
